/*
**  CXSC is a C++ library for eXtended Scientific Computing
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2001 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* CVS $Id: imath.cpp,v 1.6 2001/08/09 16:53:47 cxsc Exp $ */

#include "imath.hpp"
#include "rmath.hpp"

// Auch fi_lib hat ein eigenes interval (wie RTS sein dotprecision)
//typedef struct fi_interval { double INF, SUP;} fi_interval;
#undef LINT_ARGS
#define CXSC_INCLUDE
#include <fi_lib.h>

namespace cxsc {


interval sqr    (const interval &a) throw() 
{ 
  interval res;
  res= a*a;
  if (Inf(res)<0) Inf(res)=0;
  return res;
}

interval sqrt   (const interval &a)         { return j_sqrt(a); }

interval sqrt   (const interval &a, int n)  
{ 
   if(a!=0) // Eigentlich 0 nicht Element...
      return pow(a,interval(1.0,1.0)/n); 
   else
      return interval(0);
}

interval sin    (const interval &a) throw() { return j_sin(a); }
interval cos    (const interval &a) throw() { return j_cos(a); }
interval tan    (const interval &a) throw() { return j_tan(a); }
interval cot    (const interval &a) throw() { return j_cot(a); }

interval asin   (const interval &a)         { return j_asin(a); }
interval acos   (const interval &a)         { return j_acos(a); }
interval atan   (const interval &a)         { return j_atan(a); }
interval acot   (const interval &a)         { return j_acot(a); }

interval exp    (const interval &a) throw() { return j_exp(a); }
interval ln     (const interval &a)         { return j_log(a); }

interval sinh   (const interval &a) throw() { return j_sinh(a); }
interval cosh   (const interval &a) throw() { return j_cosh(a); }
interval tanh   (const interval &a) throw() { return j_tanh(a); }
interval coth   (const interval &a) throw() { return j_coth(a); }

interval asinh  (const interval &a)         { return j_asnh(a); }
interval acosh  (const interval &a)         { return j_acsh(a); }
interval atanh  (const interval &a)         { return j_atnh(a); }
interval acoth  (const interval &a)         { return j_acth(a); }

interval pow    (const interval &a, const interval &b) throw(ERROR_INTERVAL_STD_FKT_OUT_OF_DEF)
{
	if(Inf(a)>0)
		return j_exp(b*ln(a));
	else if(Inf(a)==0 && Inf(b)>=0)
	{
		if(Sup(a)>=1)
			return interval(0,pow(Sup(a),Sup(b)));
		else
			return interval(0,pow(Sup(a),Inf(b)));
	}
	else
	{
		cxscthrow(ERROR_INTERVAL_STD_FKT_OUT_OF_DEF("interval pow(const interval &,const interval &)"));
		return interval(0,0);
	}
}

interval power  (const interval &a, const int i)       { return pow(a,_interval(i)); }

} // namespace cxsc
