/*
**  CXSC is a C++ library for eXtended Scientific Computing
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2001 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* CVS $Id: interval.cpp,v 1.4 2001/08/09 16:53:47 cxsc Exp $ */

#include "interval.hpp"
#include "ioflags.hpp"
#include "idot.hpp"
#include "dot.hpp"
#include "RtsFunc.h" // b_shr1 in mid()

// Auch fi_lib hat ein eigenes interval (wie RTS sein dotprecision)
//typedef struct fi_interval { double INF, SUP;} fi_interval;
// class interval;
#undef LINT_ARGS
#define CXSC_INCLUDE
#include <fi_lib.h>

namespace cxsc {

interval::interval(const idotprecision & a) throw()
{
   *this=rnd(a);
}

interval & interval::operator =(const idotprecision & a) throw()
{
   return *this=rnd(a);
}

interval::interval(const dotprecision & a) throw()
{
   rnd(a,inf,sup);
}

interval::interval(const dotprecision &a,const dotprecision &b) throw(ERROR_INTERVAL_EMPTY_INTERVAL)
{
   inf=rnd(a,RND_DOWN);
   sup=rnd(b,RND_UP);
   if(inf>sup)
      cxscthrow(ERROR_INTERVAL_EMPTY_INTERVAL("interval::interval(const dotprecision &,const dotprecision &)"));   
}

interval::interval(const l_real &a,const l_real &b) throw(ERROR_INTERVAL_EMPTY_INTERVAL)
{
   dotakku[0]=a;
   inf=rnd(dotakku[0],RND_DOWN);
   dotakku[0]=b;
   sup=rnd(dotakku[0],RND_UP);
   if(inf>sup)
      cxscthrow(ERROR_INTERVAL_EMPTY_INTERVAL("interval::interval(const l_real &,const l_real &)"));   
}


interval & interval::operator =(const dotprecision & a) throw()
{
   rnd(a,inf,sup);
   return *this;
}

              
// ---- Standardfunkt ---- (arithmetische Operatoren)
/*
interval operator+(const interval &a, const interval &b) throw() { return interval(adddown(a.inf,b.inf),addup(a.sup,b.sup));  }
interval operator+(const interval &a, const real &b) throw() { return interval(adddown(a.inf,b),addup(a.sup,b));  }
interval operator+(const real &a, const interval &b) throw() { return interval(adddown(a,b.inf),addup(a,b.sup));  }

interval operator-(const interval &a, const interval &b) throw() { return interval(subdown(a.inf,b.inf),subup(a.sup,b.sup));  }
interval operator-(const interval &a, const real &b) throw() { return interval(subdown(a.inf,b),subup(a.sup,b));  }
interval operator-(const real &a, const interval &b) throw() { return interval(subdown(a,b.sup),subup(a,b.inf));  }

interval operator*(const interval &a, const interval &b) throw() { return mul_ii (a, b); }
interval operator*(const interval &a, const real &b) throw() { return mul_id (a, *(double*)&b); }
interval operator*(const real &a, const interval &b) throw() { return mul_di (*(double*)&a, b); }

interval operator/(const interval &a, const interval &b) throw() {  return div_ii (a, b); }
interval operator/(const interval &a, const real &b) throw() {  return div_id (a, *(double*)&b); }
interval operator/(const real &a, const interval &b) throw() {  return div_di (*(double*)&a, b); }
*/


bool operator ==(const interval &a,const dotprecision &r)     throw() {   return(r==a.inf && r==a.sup); }
bool operator !=(const interval &a,const dotprecision &r)     throw() {   return(r!=a.inf || r!=a.sup); }
bool operator ==(const dotprecision &r,const interval &a)     throw() {   return(r==a.inf && r==a.sup); }
bool operator !=(const dotprecision &r,const interval &a)     throw() {   return(r!=a.inf || r!=a.sup); }

bool operator <=(const dotprecision &a,const interval &b) throw()
{
   return(a>=b.inf && a<=b.sup);   
}
bool operator >=(const dotprecision &a,const interval &b) throw()
{
   return(a<=b.inf && a>=b.sup);   
}
bool operator <(const dotprecision &a,const interval &b) throw()
{
   return(a>b.inf && a<b.sup);   
}

bool operator <=(const interval &a,const dotprecision &b) throw()
{
   return(a.inf>=b && a.sup<=b);   
}
bool operator >=(const interval &a,const dotprecision &b) throw()
{
   return(a.inf<=b && a.sup>=b);   
}
bool operator >(const interval &a,const dotprecision &b) throw()
{
   return(a.inf<b && a.sup>b);   
}

interval operator+ (const interval& a, const interval& b) throw()
  { return interval (adddown(a.inf,b.inf),addup(a.sup,b.sup)); }
interval operator+ (const interval& a, const real& b) throw() 
  { return interval (adddown(a.inf,b),addup(a.sup,b)); }
interval operator+ (const real& a, const interval& b) throw() 
  { return interval (adddown(a,b.inf),addup(a,b.sup)); }

interval operator-  (const interval& a, const interval& b) throw() 
  { return interval ( subdown(a.inf,b.sup), subup(a.sup,b.inf)); }
interval operator-  (const interval& a, const real& b) throw() 
  { return interval ( subdown(a.inf,b), subup(a.sup,b)); }
interval operator-  (const real& a, const interval& b) throw() 
  { return interval ( subdown(a,b.sup), subup(a,b.inf)); }

  //------------------------------------------------------------------------
  //  a * b  = ueber Entscheidungstabelle :
  //
  //                      bi,bs >= 0       bi < 0, bs >=0       bi,bs < 0
  // ----------------+------------------+------------------+----------------
  // ai,as >= 0      I   ai*bi, as*bs   I   as*bi, as*bs   I   as*bi, ai*bs
  // ----------------+------------------+------------------+----------------
  // ai < 0, as >= 0 I   ai*bs, as*bs   I      ....        I   as*bi, ai*bi
  // ----------------+------------------+------------------+----------------
  // ai,as < 0       I   ai*bs, as*bi   I   ai*bs, ai*bi   I   as*bs, ai*bi
  // ----------------+------------------+------------------+----------------
  //
  //  .... :  min(ai*bs, as*bi), max(ai*ai, as*as)
  //
interval operator *(const interval& a, const interval& b) throw()
{
   interval tmp;

   if (sign(a.inf) >= 0) 
   {                                   // 1. Zeile der Entscheidungstabelle
      if (sign(b.inf) >= 0)
      {                                  //     1. Spalte: [ai*bi, as*bs]
         tmp.inf = multdown(a.inf,b.inf);
         tmp.sup = multup(a.sup,b.sup);
      } else if (sign(b.sup) >= 0) 
      {                                  //     2. Spalte: [as*bi, as*bs]
         tmp.inf = multdown(a.sup,b.inf);
         tmp.sup = multup(a.sup,b.sup);
      } else 
      {                                  //     3. Spalte: [as*bi, ai*bs]
         tmp.inf = multdown(a.sup,b.inf);
         tmp.sup = multup(a.inf,b.sup);
      }
   } else if (sign(a.sup) >= 0) 
   {                                   // 2. Zeile der Entscheidungstabelle
      if (sign(b.inf) >= 0) 
      {                                  //     1. Spalte: [ai*bs, as*bs]
         tmp.inf = multdown(a.inf,b.sup);
         tmp.sup = multup(a.sup,b.sup);
      } else if (sign(b.sup) >= 0) 
      {                                  //   2. Spalte: [min(ai*bs, as*bi),
         real hlp;                       //                 max(ai*ai, as*as)]

         tmp.inf = multdown(a.inf,b.sup);
         hlp     = multdown(a.sup,b.inf);

         if (hlp < tmp.inf) 
            tmp.inf = hlp;

         tmp.sup = multup(a.inf,b.inf);
         hlp     = multup(a.sup,b.sup);
         
         if (hlp > tmp.sup) 
            tmp.sup = hlp;               
      } else 
      {                                  //     3. Spalte: [as*bi, ai*bi]
         tmp.inf = multdown(a.sup,b.inf);
         tmp.sup = multup(a.inf,b.inf);
      }
   } else 
   {                                   // 3. Zeile der Entscheidungstabelle
      if (sign(b.inf) >= 0) 
      {                                  //     1. Spalte: [ai*bs, as*bi]
         tmp.inf = multdown(a.inf,b.sup);
         tmp.sup = multup(a.sup,b.inf);
      } else if (sign(b.sup) >= 0) 
      {                                  //   2. Spalte: [ai*bs, ai*bi]
         tmp.inf = multdown(a.inf,b.sup);
         tmp.sup = multup(a.inf,b.inf);
      } else 
      {                                  //     3. Spalte: [as*bs, ai*bi]
         tmp.inf = multdown(a.sup,b.sup);
         tmp.sup = multup(a.inf,b.inf);
      }
   }
   return tmp;
}

  //------------------------------------------------------------------------
        //  a / b  = ueber Entscheidungstabelle :
  //
  //                      bi,bs > 0          bi,bs < 0
  // ----------------+------------------+------------------
  // ai,as >= 0      I   ai/bs, as/bi   I   as/bs, ai/bi
  // ----------------+------------------+------------------
  // ai < 0, as >= 0 I   ai/bi, as/bi   I   as/bs, ai/bs
  // ----------------+------------------+------------------
  // ai,as < 0       I   ai/bi, as/bs   I   as/bi, ai/bs
  // ----------------+------------------+------------------
  //
interval operator/  (const interval& a, const interval& b) throw()
{
   interval tmp;
   /* if (!b) 
   {
      errmon (ERR_INTERVAL(DIVZERO));
      tmp.inf = (!a) ? QuietNaN : Infinity;  // mr????
      tmp.sup = tmp.inf;
      return tmp;
   }*/                                                  // ERRORHANDLING?!?

   if (sign(a.inf) >= 0) 
   {                                    // 1. Zeile der Entscheidungstabelle
      if (sign(b.inf) > 0) 
      {                                 //     1. Spalte: [ai/bs, as/bi]
         tmp.inf = divdown(a.inf,b.sup);
         tmp.sup = divup(a.sup,b.inf);
      } else 
      {                                 //     2. Spalte: [as/bs, ai/bi]
         tmp.inf = divdown(a.sup,b.sup);
         tmp.sup = divup(a.inf,b.inf);
      }
   } else if (sign(a.sup) >= 0) 
   {                                    // 2. Zeile der Entscheidungstabelle
      if (sign(b.inf) > 0) 
      {                                 //     1. Spalte: [ai/bi, as/bi]
         tmp.inf = divdown(a.inf,b.inf);
         tmp.sup = divup(a.sup,b.inf);
      } else 
      {                                 //     2. Spalte: [as/bs, ai/bs]
         tmp.inf = divdown(a.sup,b.sup);
         tmp.sup = divup(a.inf,b.sup);
      }
   } else 
   {                                    // 3. Zeile der Entscheidungstabelle
      if (sign(b.inf) > 0) 
      {                                 //     1. Spalte: [ai/bi, as/bs]
         tmp.inf = divdown(a.inf,b.inf);
         tmp.sup = divup(a.sup,b.sup);
      } else 
      {                                 //     2. Spalte: [as/bi, ai/bs]
         tmp.inf = divdown(a.sup,b.inf);
         tmp.sup = divup(a.inf,b.sup);
      }
   }

   return tmp;
}

interval operator*  (const real& a, const interval& b) throw()
{
   interval tmp;
   if (sign(a) == 0) 
      tmp = _interval(0.0,0.0);
   else if (sign(a) > 0) 
   {
      tmp.inf = multdown(a,b.inf);
      tmp.sup = multup(a,b.sup);
   } else if (sign(a) < 0) 
   {
      tmp.inf = multdown(a,b.sup);
      tmp.sup = multup(a,b.inf);
   }
   return tmp;
}

interval operator*  (const interval& a, const real& b) throw()
{
   interval tmp;
   if (sign(b) == 0) 
      tmp = _interval(0.0,0.0);
   else if (sign(b) > 0) 
   {
      tmp.inf = multdown(a.inf,b);
      tmp.sup = multup(a.sup,b);
   } else if (sign(b) < 0) 
   {
      tmp.inf = multdown(a.sup,b);
      tmp.sup = multup(a.inf,b);
   }
   return tmp;
}

interval operator/  (const real& a, const interval& b)  throw() { return (_interval(a) / b); }
interval operator/  (const interval& a, const real& b)  throw() { return (a / _interval(b)); }


// ---- Ausgabefunkt. ---------------------------------------

std::ostream & operator << (std::ostream &s, const interval& a) throw()
{
   s << '['          << SaveOpt << RndDown
     << a.inf << ',' << RndUp 
     << a.sup        << RestoreOpt 
     << ']';
   return s;
}
std::string & operator << (std::string &s, const interval &a) throw()
{
   s+='[';
   s << SaveOpt << RndDown
     << a.inf;
   s+=',';
   s << RndUp 
     << a.sup   << RestoreOpt; 
   s+=']';
   return s;
}

std::istream & operator >> (std::istream &s, interval &a) throw()
{
   char c;

   skipeolnflag = inpdotflag = true;
   c = skipwhitespacessinglechar (s, '[');
   if (inpdotflag) 
      s.putback(c);

   s >> SaveOpt >> RndDown >> a.inf;

   skipeolnflag = inpdotflag = true;
   c = skipwhitespacessinglechar (s, ',');
   if (inpdotflag) s.putback(c);

   s >> RndUp >> a.sup >> RestoreOpt;

   if (!waseolnflag) 
   {
      skipeolnflag = false, inpdotflag = true;
      c = skipwhitespaces (s);
      if (inpdotflag && c != ']') 
         s.putback(c);
   }

   /*if (a.inf > a.sup) {
      errmon (ERR_INTERVAL(EMPTY));
   } */         
   return s;
}

std::string & operator >> (std::string &s, interval &a) throw()
{
   s = skipwhitespacessinglechar (s, '[');
   s >> SaveOpt >> RndDown >> a.inf;
   s = skipwhitespacessinglechar (s, ',');
   s >> RndUp >> a.sup >> RestoreOpt;
   s = skipwhitespaces (s);

   if (s[0] == ']') 
      s.erase(0,1);

    /*if (a.inf > a.sup) {
      errmon (ERR_INTERVAL(EMPTY));
    } */                         
   return s;
}

void operator >>(const std::string &s,interval &a) throw()
{
   std::string r(s);
   r>>a;
}
void operator >>(const char *s,interval &a) throw()
{
   std::string r(s);
   r>>a;
}

real mid(const interval & a) throw()
{
   dotakku[4] =  a.inf;
   dotakku[4] += a.sup;
    
   if (dotakku[4] != 0.0) 
   {
      // Division nur bei ungleich 0.0
      Dotprecision ptr = *dotakku[4].ptr();
                   
      // --------------------------------------------------------
      //  Dividiere dotakku[0] durch 2, mittels 1 Rechtsshift
                               
      ptr[(a_intg)++ptr[A_END]] = ZERO;
      
      b_shr1 (&ptr[(a_intg)ptr[A_BEGIN]], (a_intg)(ptr[A_END]-ptr[A_BEGIN]+1));
                                                   
      if (ptr[(a_intg)ptr[A_END]]   == ZERO) 
         --ptr[A_END];
                                                         
      if (ptr[(a_intg)ptr[A_BEGIN]] == ZERO) 
         ++ptr[A_BEGIN];   

      // --------------------------------------------------------
   }
          
   return rnd(dotakku[4]);
}

} // namespace cxsc
