/*
**  CXSC is a C++ library for eXtended Scientific Computing
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2001 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* CVS $Id: l_rmath.cpp,v 1.3 2001/08/09 11:37:57 cxsc Exp $ */

#include "l_rmath.hpp"
#include "rmath.hpp"

namespace cxsc {

l_real sqrt(const l_real &x) throw(ERROR_LREAL_STD_FKT_OUT_OF_DEF)
{
   int       stagsave = stagprec,stagmax = 19,stagsave2;
   l_real    a1, y, t, mt;
   if (x < 0.0) 
      cxscthrow(ERROR_LREAL_STD_FKT_OUT_OF_DEF("l_real sqrt(const l_real &x)"));
   else if (x == 0.0 || x == 1.0 )
      y = x;
   else 
   {
       // Einsatz des Newton-Verfahrens y = y-f(y)/f'(y)
       if (stagprec < stagmax) stagsave2 = stagprec+1;  
       else                    stagsave2 = stagmax;
       y = sqrt(_real(x));
       stagprec = 1; 
       while (stagprec < stagsave2) 
       {
          stagprec += stagprec;
          if (stagprec > stagsave2) 
             stagprec = stagsave2;
          y = 0.5*((x/y)+y);
       }
       stagprec = stagsave;
       adjust(y);
   }
   return y;
}

l_real power(const l_real& x, int n)
{
   int       stagsave = stagprec, stagmax = 19;
   long int  zhi = 2;
   l_real    y, neu;
   bool      neg=false;

   if (x == 1.0)
      y = x;
   else if (n == 0)
      y = adjust(_l_real(1.0));
   else 
   {
      if (stagprec < stagmax) 
         stagprec++;
      else
         stagprec = stagmax;

      if (n == 1)
         y = x;
      else if (n == 2)
         y = x*x;
      else 
      {
         if (n < 0) 
         {
            neg = true;
            n = -n;
         }

         // Initialisierung
         if (n%2)   
            y = x;
         else      
            y = 1.0;  // Praezision wird bei 1 Mult. auf
                      // aktuellen Wert gesetzt;
                      // Berechnung durch binaere Darstellung der n
         neu = x*x;
         do {
            if ((n/zhi)%2)
               y *= neu;
            zhi += zhi;
            if (zhi <= n)  // letzte Mult. entfaellt --> schneller
               neu *= neu;
         } while (zhi <= n);
         if (neg) 
            y = 1/(y);
      }
      stagprec = stagsave;
      y = adjust(y);
   }
   return y;
}

} // namespace cxsc
